%% Empirical application 

clear ; 
clc; 

%*******************************************************************************
%                          Loading the data set
%*******************************************************************************


dataset = xlsread('final_data_k401.xls') ;

dataset = dataset(dataset(:,7) >= quantile(dataset(:,7),0.01) & dataset(:,7) <= quantile(dataset(:,7),0.99), :); 
n = size(dataset,1) ; 

income = dataset(:,2) ; 
marital = dataset(:,3) ; 
male = dataset(:,4) ; 
age = dataset(:,5) ; 
fam_size = dataset(:,6); 
nettfa = dataset(:,7) ; % Y variable
p401k = dataset(:,8) ; 


% Standardizing X variable 
std_inc = (income - mean(income)) / std(income) ; 
%std_educ = (meduc - mean(meduc)) / std(meduc) ; 
%std_age = (age - mean(age)) / std(age) ; 


y = nettfa/10000; 
d = p401k; 

x = std_inc ; 

mean_x = mean(x) ; 
std_x = std(x) ; 

dx = size(x, 2) ; 

data = [y,d,x]; 


%% Supports 
y_inc = 0.1 ; 

y1_sub = y(d == 1); 
y0_sub = y(d == 0); 

y1_max = max(y1_sub); 
y0_max = max(y0_sub); 
y1_min = min(y1_sub); 
y0_min = min(y0_sub); 

yl = y1_min - y0_max; 
yu = y1_max - y0_min;

y_index1 = (yl:y_inc:0)'; 
y_index2 = (0:y_inc:yu)'; 
y_index = [y_index1 ; y_index2]; 

%y_index = (yl:y_inc:yu)' ; 
di = size(y_index,1) ; 
delta_index = y_index; 

q_index = (0:0.1:0.9)'; 


lb_all_mat = zeros(di, size(q_index,1)) ; 
ub_all_mat = zeros(di, size(q_index,1)) ; 

tight_lb_all_mat = zeros(di, size(q_index,1)) ; 
tight_ub_all_mat = zeros(di, size(q_index,1)) ; 

cv_lower_all_mat = zeros(1, size(q_index,1)) ; 
cv_upper_all_mat = zeros(1, size(q_index,1)) ; 

rate_store_mat = zeros(1,size(q_index,1)) ; 



for qq = 1: size(q_index,1) 

qx_level = q_index(qq,1)  ; 


if qx_level == 0     
    x_pt_est = mean(x) ; 
else 
    x_pt_est = quantile(x,qx_level) ; 
    
end


% Bandwidth and point of x

cbw = 1.06 * std_x ; 
bw_order = (-1/(5+dx)) ; 
bw = n^(bw_order) * cbw ; 

%% Storing matrices 

tight_bound_est_mat  = zeros(di,2) ; 
bound_est_mat = zeros(di,2) ; 



%% Bounds on Dist of TE

for i = 1:di  
    
    bound_est_mat(i,:) = FH_bound_endo(data, delta_index(i,1), y_index, x_pt_est, bw, 0) ; 
    tight_bound_est_mat(i,:) = FH_bound_endo(data, delta_index(i,1), y_index, x_pt_est, bw, 1) ; % Assumptions 2.3 and 2.4 are imposed 
   
end

lb_est_mat = bound_est_mat(:,1) ; 
ub_est_mat = bound_est_mat(:,2) ; 

tight_lb_est_mat = tight_bound_est_mat(:,1) ; 
tight_ub_est_mat = tight_bound_est_mat(:,2) ; 


lb_all_mat(:,qq) = lb_est_mat; 
ub_all_mat(:,qq) = ub_est_mat; 

tight_lb_all_mat(:,qq) = tight_lb_est_mat; 
tight_ub_all_mat(:,qq) = tight_ub_est_mat; 


%% Results

qx = qx_level * 100; 

save(sprintf('Empirical_Results_Endo_Final_qx_%d_dx_%d.mat',qx, dx),'n','qx','bw', 'lb_est_mat', 'ub_est_mat', 'lb_est_mat', 'ub_est_mat', 'tight_lb_est_mat', 'tight_ub_est_mat', 'delta_index', 'data', 'dx');     


end

save(sprintf('Empirical_Results_Endo_Final_dx_%d.mat', dx), 'n', 'dx', 'delta_index', 'lb_all_mat', 'ub_all_mat', 'tight_lb_all_mat', 'tight_ub_all_mat') ; 


%% 
%*******************************************************************************
%        Post-Estimation Graph Illustration
%*******************************************************************************

load('Empirical_Results_Endo_Final_dx_1.mat'); %conditional on income 

%% Comparison with and without SD 


q = 0.5; 
q_col = q*10 + 1; 


plot(delta_index, lb_all_mat(:,q_col), 'b--', delta_index, ub_all_mat(:,q_col), 'r--', delta_index, tight_lb_all_mat(:,q_col), 'b', delta_index, tight_ub_all_mat(:,q_col), 'r') 
xlim([min(delta_index), max(delta_index)]) 
ylim([-0.5 1.5])
legend('LB', 'UB', 'LB under SD', 'UB under SD', 'location', 'northwest') ; 


%% Comparison across quantiles

q_low = 0.2; 
q_high = 0.8 ; 

q_low_col = q_low*10 + 1; 
q_high_col = q_high*10 + 1; 

plot(delta_index, tight_lb_all_mat(:,q_low_col), 'b-*', delta_index, tight_ub_all_mat(:,q_low_col), 'r-*', delta_index, tight_lb_all_mat(:,q_high_col), 'b-o', delta_index, tight_ub_all_mat(:,q_high_col), 'r-o')
xlim([min(delta_index), max(delta_index)]) 
legend('LB at low', 'UB at low', 'LB at high', 'UB at high', 'location','northwest') ; 


%% Comparison with at the mean 

q_mean = 0; 
q_level = 0.8 ; 

mean_col = q_mean*10 + 1; 
q_level_col = q_level*10 + 1; 

plot(delta_index, tight_lb_all_mat(:,mean_col), 'b-*', delta_index, tight_ub_all_mat(:,mean_col), 'r-*', delta_index, tight_lb_all_mat(:,q_level_col), 'b-o', delta_index, tight_ub_all_mat(:,q_level_col), 'r-o')
xlim([min(delta_index), max(delta_index)]) 
legend('LB at mean', 'UB at mean', 'LB at quantile', 'UB at quantile', 'location','northwest') ; 
