%% Unconfoundedness Simulation: Dist of TE 
%% KS statistic, an is of a sqrt-log rate 

clear; 
clc; 

tic; 

n = 500; 
m = 500; 
bb = 500 ; % number of bootstrap iterations 

anc_mat = [0.1; 0.2; 0.3; 0.4; 0.5]; 

for aa = 1:size(anc_mat,1)  

anc = anc_mat(aa,1); 

%% Parameters 
loc1 = 1; % change this value to consider various H1
loc0 = 0; 

loc = [loc1; loc0]; 

beta1 = 1; 
beta0 = 0.8 ; 

beta = [beta1; beta0];

gamma1 = 1 ; 
gamma0 = 0.7 ; 

gamma = [gamma1; gamma0]; % scale in scale parameter

phi1 = 1 ; 
phi0 = 1 ; %location in scale parameter

phi = [phi1; phi0];

alpha = 1; 

rho = 0; 
mu_u = zeros(2,1) ;
sigma_u = [1,rho;rho,1]; 

mu_x = 0 ; 


level = 0.05 ; %significance level 

qx = 0; % Choosing quantile of x ; if qx=0, we calculate at the mean of x
qx100 = qx*100 ; 


cv_store_lower = zeros(m,1) ; 
decision_mat = zeros(m,1) ; 
bw_store_mat = zeros(m,1) ; 
ks_stat_mat = zeros(m,1) ; 



%% DGP and simulation
parfor sim = 1:m  


u = mvnrnd(mu_u, sigma_u,n) ;
u1 = u(:,1) ;
u0 = u(:,2) ;

v = normrnd(0,1,n,1) ;

x = rand(n,1)*2 - 1; 



d = (x*alpha >= v) ;
y1 = loc1 + x*beta1 + (phi1 + (x*gamma1).^2) .* u1; 
y0 = loc0 + x*beta0 + (phi0 + (x*gamma0).^2) .* u0; 


y = d.*y1 + (1-d).*y0 ; 

data = [y,d,x] ;

te = y1 - y0 ; 




%% Choosing x_pt and bandwidth



if qx == 0 
    x_pt_true = mu_x ; %mean of X
    
end


x_pt_est = x_pt_true; 
x_pt_true = unifinv(qx)*2 - 1; 
%x_pt_est = quantile(x,qx) ;

 

mu1 = loc1 + x_pt_true * beta1 ; 
var_y1 = (phi1 + (x_pt_true*gamma1)^2)^2 * sigma_u(1,1)^2 ; 
sd1 = sqrt(var_y1) ;

mu0 = loc0 + x_pt_true * beta0 ; 
var_y0 = (phi0 + (x_pt_true*gamma0)^2)^2 * sigma_u(2,2)^2 ; 
sd0 = sqrt(var_y0) ;



cbw = 1.06*std(x) ; 
%cbw = 1; 

bw_order = -1/6; 
hn = n^bw_order ; 

bw = hn * cbw ; % bandwidth

bw_store_mat(sim,1) = bw; 


%% Supports 

y1_sub = y(d == 1); 
y0_sub = y(d == 0); 

y1_max = max(y1_sub); 
y0_max = max(y0_sub); 
y1_min = min(y1_sub); 
y0_min = min(y0_sub); 

yl = y1_min - y0_max; 
yu = y1_max - y0_min;

y_index = (yl:0.05:yu)' ; 
di = size(y_index,1) ; 
delta_index = y_index; 



%% Kernel Estimation of marginal distributions

f1_est_mat = zeros(di,1) ; 
f0_est_mat = zeros(di,1) ; 

for i = 1:di 
    
    f_est_temp = f_est(data, y_index(i,1), 0, x_pt_est, bw) ;
    
    f1_est_mat(i,1) = f_est_temp(1) ;
    f0_est_mat(i,1) = f_est_temp(2) ;
    
end



%% Bounds on Dist of TE

bound_est_mat  = zeros(di,2) ; 
bound_true_mat = zeros(di,2) ; % hypothesized true bound

for i = 1:di    
   
    bound_est_mat(i,:) = FH_bound(data, delta_index(i,1), y_index, x_pt_est, bw) ; 
    %bound_true_mat(i,:) = true_bound(delta_index(i,1), mu1, mu0, sd1, sd0) ; 
    bound_true_mat(i,:) = true_bound(delta_index(i,1), 0, 0, 1, 1) ; 
   
end

lb_est_mat = bound_est_mat(:,1) ; 
ub_est_mat = bound_est_mat(:,2) ; 

lb_true_mat = bound_true_mat(:,1) ; 
ub_true_mat = bound_true_mat(:,2) ; 

%% KS statistic

ks = max(abs(lb_est_mat - lb_true_mat)) ; 
ks_stat_mat(sim,1) = sqrt(n*bw) * ks; 


%% Boostrap 

%an = anc * log(log(n*bw))/sqrt(n*bw) ; 

an = anc * sqrt(log(n*bw))/sqrt(n*bw) ; 

lb_te_boot_mat = zeros(bb,1) ;
ub_te_boot_mat = zeros(bb,1) ;

for b = 1:bb 
    boot = normrnd(0,1,n,1) ;

    bound_boot = bound_te_boot(data, y_index, delta_index, x_pt_est, n, bw, boot, an) ; 
    lb_te_boot_mat(b,1) = bound_boot(1) ;
    ub_te_boot_mat(b,1) = bound_boot(2) ; 
    
end

cv_lower = quantile(lb_te_boot_mat, 1-level) ; 
cv_store_lower(sim,1) = cv_lower ; 

decision_mat(sim,1) = (ks_stat_mat(sim,1) >= cv_lower) ; 



end

%mean(decision_mat)

ancp = anc*100 ; 


save(sprintf('Final_TE_Simulation_alt_%d_%d_%d_%d_%d_loc1_an_sqrt_log_rate.mat',n,m,bb,qx100,ancp),'n','m','bb','qx100','level','cv_store_lower', 'ks_stat_mat', 'decision_mat', 'bw_store_mat', 'beta', 'loc', 'gamma', 'phi', 'alpha', 'anc'); 

end


%plot(delta_index, lb_est_mat, delta_index, lb_true_mat)
%legend('est', 'true'); 